@@ -12,7 +12,7 @@ from typing import Optional
 import websockets
 
 from mirai import exceptions
-from mirai.adapters.base import Adapter, error_handler_async, json_dumps
+from mirai.adapters.base import Adapter, AdapterInterface, error_handler_async, json_dumps
 from mirai.api_provider import Method
 from mirai.tasks import Tasks
 
@@ -53,7 +53,10 @@ class WebSocketAdapter(Adapter):
 
         `single_mode: bool = False` 是否启用单例模式。
         """
-        super().__init__(verify_key=verify_key)
+        super().__init__(verify_key=verify_key, single_mode=single_mode)
+
+        self._host = host
+        self._port = port
 
         if host[:2] == '//':
             host = 'ws:' + host
@@ -76,9 +79,34 @@ class WebSocketAdapter(Adapter):
         self._recv_dict = defaultdict(deque)
         # 本地同步 ID，每次调用 API 递增。
         self._local_sync_id = random.randint(1, 1024) * 1024
-        #
+        # 事件处理任务管理器
         self._tasks = Tasks()
 
+    @property
+    def adapter_info(self):
+        return {
+            'verify_key': self.verify_key,
+            'session': self.session,
+            'single_mode': self.single_mode,
+            'host': self._host,
+            'port': self._port,
+            'sync_id': self.sync_id,
+        }
+
+    @classmethod
+    def via(cls, adapter_interface: AdapterInterface) -> "WebSocketAdapter":
+        info = adapter_interface.adapter_info
+        adapter = cls(
+            verify_key=info['verify_key'],
+            **{
+                key: info[key]
+                for key in ['host', 'port', 'sync_id', 'single_mode']
+                if info.get(key) is not None
+            }
+        )
+        adapter.session = info.get('session')
+        return adapter
+
     @_error_handler_async_local
     async def _receiver(self):
         """开始接收 websocket 数据。"""
@@ -117,10 +145,19 @@ class WebSocketAdapter(Adapter):
 
     async def _recv(self, sync_id: str = '-1') -> dict:
         """接收并解析 websocket 数据。"""
-        while not self._recv_dict[sync_id]:
-            # 如果没有对应同步 ID 的数据，则等待 websocket 数据
-            await asyncio.sleep(0.1)
-        return self._recv_dict[sync_id].popleft()
+        for _ in range(100):
+            if self._recv_dict[sync_id]:
+                return self._recv_dict[sync_id].popleft()
+            else:
+                # 如果没有对应同步 ID 的数据，则等待 websocket 数据
+                # 目前存在问题：如果 mah 发回的数据不含 sync_id，
+                # 这里就会无限循环……
+                # 所以还是限制 100 次好了。
+                await asyncio.sleep(0.1)
+        else:
+            raise TimeoutError(
+                f'[WebSocket] mirai-api-http 响应超时，可能是由于调用出错。同步 ID：{sync_id}。'
+            )
 
     @_error_handler_async_local
     async def login(self, qq: int):
@@ -171,6 +208,10 @@ class WebSocketAdapter(Adapter):
             content['subCommand'] = 'get'
         elif method == Method.RESTPOST:
             content['subCommand'] = 'update'
+        elif method == Method.MULTIPART:
+            raise NotImplementedError(
+                "WebSocket 适配器不支持上传操作。请使用 bot.use_adapter 临时调用 HTTP 适配器。"
+            )
 
         await self.connection.send(json_dumps(content))
         logger.debug(f"[WebSocket] 发送 WebSocket 数据，同步 ID：{sync_id}。")
